@@ -19,47 +19,48 @@
 namespace xt
 {
 
-    /**********************
-     * xarray declaration *
-     **********************/
+    /********************************
+     * xarray_container declaration *
+     ********************************/
 
-    template <class T, class EA, class SA>
-    struct xcontainer_inner_types<xarray<T, EA, SA>>
+    template <class EC, class SC>
+    struct xcontainer_inner_types<xarray_container<EC, SC>>
     {
-        using container_type = std::vector<T, EA>;
-        using shape_type = std::vector<typename container_type::size_type, SA>;
+        using container_type = EC;
+        using shape_type = SC;
         using strides_type = shape_type;
         using backstrides_type = shape_type;
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
-        using temporary_type = xarray<T, EA, SA>;
+        using temporary_type = xarray_container<EC, SC>;
     };
 
-    template <class T, class EA, class SA>
-    struct xiterable_inner_types<xarray<T, EA, SA>>
-        : xcontainer_iterable_types<xarray<T, EA, SA>>
+    template <class EC, class SC>
+    struct xiterable_inner_types<xarray_container<EC, SC>>
+        : xcontainer_iterable_types<xarray_container<EC, SC>>
     {
     };
 
     /**
-     * @class xarray
-     * @brief Dense multidimensional container with tensor
-     * semantic.
+     * @class xarray_container
+     * @brief Dense multidimensional container with tensor semantic.
      *
-     * The xarray class implements a dense multidimensional container
+     * The xarray_container class implements a dense multidimensional container
      * with tensor semantic.
      *
-     * @tparam T The type of objects stored in the container.
+     * @tparam EC The type of the container holding the elements.
+     * @tparam SC The type of the containers holding the shape and the strides.
+     * @sa xarray
      */
-    template <class T, class EA, class SA>
-    class xarray : public xstrided_container<xarray<T, EA, SA>>,
-                   public xcontainer_semantic<xarray<T, EA, SA>>
+    template <class EC, class SC>
+    class xarray_container : public xstrided_container<xarray_container<EC, SC>>,
+                             public xcontainer_semantic<xarray_container<EC, SC>>
     {
 
     public:
 
-        using self_type = xarray<T, EA, SA>;
+        using self_type = xarray_container<EC, SC>;
         using base_type = xstrided_container<self_type>;
         using semantic_base = xcontainer_semantic<self_type>;
         using container_type = typename base_type::container_type;
@@ -71,32 +72,32 @@ namespace xt
         using shape_type = typename base_type::shape_type;
         using strides_type = typename base_type::strides_type;
 
-        xarray();
-        explicit xarray(const shape_type& shape, layout l = layout::row_major);
-        explicit xarray(const shape_type& shape, const_reference value, layout l = layout::row_major);
-        explicit xarray(const shape_type& shape, const strides_type& strides);
-        explicit xarray(const shape_type& shape, const strides_type& strides, const_reference value);
+        xarray_container();
+        explicit xarray_container(const shape_type& shape, layout l = layout::row_major);
+        explicit xarray_container(const shape_type& shape, const_reference value, layout l = layout::row_major);
+        explicit xarray_container(const shape_type& shape, const strides_type& strides);
+        explicit xarray_container(const shape_type& shape, const strides_type& strides, const_reference value);
 
-        xarray(const value_type& t);
-        xarray(nested_initializer_list_t<T, 1> t);
-        xarray(nested_initializer_list_t<T, 2> t);
-        xarray(nested_initializer_list_t<T, 3> t);
-        xarray(nested_initializer_list_t<T, 4> t);
-        xarray(nested_initializer_list_t<T, 5> t);
+        xarray_container(const value_type& t);
+        xarray_container(nested_initializer_list_t<value_type, 1> t);
+        xarray_container(nested_initializer_list_t<value_type, 2> t);
+        xarray_container(nested_initializer_list_t<value_type, 3> t);
+        xarray_container(nested_initializer_list_t<value_type, 4> t);
+        xarray_container(nested_initializer_list_t<value_type, 5> t);
 
-        ~xarray() = default;
+        ~xarray_container() = default;
 
-        xarray(const xarray&) = default;
-        xarray& operator=(const xarray&) = default;
+        xarray_container(const xarray_container&) = default;
+        xarray_container& operator=(const xarray_container&) = default;
 
-        xarray(xarray&&) = default;
-        xarray& operator=(xarray&&) = default;
+        xarray_container(xarray_container&&) = default;
+        xarray_container& operator=(xarray_container&&) = default;
 
         template <class E>
-        xarray(const xexpression<E>& e);
+        xarray_container(const xexpression<E>& e);
 
         template <class E>
-        xarray& operator=(const xexpression<E>& e);
+        xarray_container& operator=(const xexpression<E>& e);
 
     private:
 
@@ -105,32 +106,32 @@ namespace xt
         container_type& data_impl() noexcept;
         const container_type& data_impl() const noexcept;
 
-        friend class xcontainer<xarray<T, EA, SA>>;
+        friend class xcontainer<xarray_container<EC, SC>>;
     };
 
     /******************************
      * xarray_adaptor declaration *
      ******************************/
 
-    template <class C, class EA = std::allocator<typename C::value_type>, class SA = std::allocator<typename C::size_type>>
+    template <class EC, class SC = std::vector<typename EC::size_type>>
     class xarray_adaptor;
 
-    template <class C, class EA, class SA>
-    struct xcontainer_inner_types<xarray_adaptor<C, EA, SA>>
+    template <class EC, class SC>
+    struct xcontainer_inner_types<xarray_adaptor<EC, SC>>
     {
-        using container_type = C;
-        using shape_type = std::vector<typename container_type::size_type, SA>;
+        using container_type = EC;
+        using shape_type = SC;
         using strides_type = shape_type;
         using backstrides_type = shape_type;
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
-        using temporary_type = xarray<typename C::value_type, EA, SA>;
+        using temporary_type = xarray_container<EC, SC>;
     };
 
-    template <class C, class EA, class SA>
-    struct xiterable_inner_types<xarray_adaptor<C, EA, SA>>
-        : xcontainer_iterable_types<xarray_adaptor<C, EA, SA>>
+    template <class EC, class SC>
+    struct xiterable_inner_types<xarray_adaptor<EC, SC>>
+        : xcontainer_iterable_types<xarray_adaptor<EC, SC>>
     {
     };
 
@@ -144,16 +145,17 @@ namespace xt
      * a multidimensional container semantic and a tensor semantic to
      * stl-like containers.
      *
-     * @tparam C The container type to adapt.
+     * @tparam EC The container type to adapt.
+     * @tparam SC The type of the containers holding the shape and the strides.
      */
-    template <class C, class EA, class SA>
-    class xarray_adaptor : public xstrided_container<xarray_adaptor<C, EA, SA>>,
-                           public xadaptor_semantic<xarray_adaptor<C, EA, SA>>
+    template <class EC, class SC>
+    class xarray_adaptor : public xstrided_container<xarray_adaptor<EC, SC>>,
+                           public xadaptor_semantic<xarray_adaptor<EC, SC>>
     {
 
     public:
 
-        using self_type = xarray_adaptor<C, EA, SA>;
+        using self_type = xarray_adaptor<EC, SC>;
         using base_type = xstrided_container<self_type>;
         using semantic_base = xadaptor_semantic<self_type>;
         using container_type = typename base_type::container_type;
@@ -185,49 +187,49 @@ namespace xt
         using temporary_type = typename xcontainer_inner_types<self_type>::temporary_type;
         void assign_temporary_impl(temporary_type& tmp);
 
-        friend class xcontainer<xarray_adaptor<C, EA, SA>>;
-        friend class xadaptor_semantic<xarray_adaptor<C, EA, SA>>;
+        friend class xcontainer<xarray_adaptor<EC, SC>>;
+        friend class xadaptor_semantic<xarray_adaptor<EC, SC>>;
     };
 
-    /*************************
-     * xarray implementation *
-     *************************/
+    /***********************************
+     * xarray_container implementation *
+     ***********************************/
 
     /**
      * @name Constructors
      */
     //@{
     /**
-     * Allocates an uninitialized xarray that holds 0 element.
+     * Allocates an uninitialized xarray_container that holds 0 element.
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray()
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container()
         : base_type(), m_data(1, value_type())
     {
     }
 
     /**
-     * Allocates an uninitialized xarray with the specified shape and
+     * Allocates an uninitialized xarray_container with the specified shape and
      * layout.
-     * @param shape the shape of the xarray
-     * @param l the layout of the xarray
+     * @param shape the shape of the xarray_container
+     * @param l the layout of the xarray_container
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(const shape_type& shape, layout l)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(const shape_type& shape, layout l)
         : base_type()
     {
         base_type::reshape(shape, l);
     }
 
     /**
-     * Allocates an xarray with the specified shape and layout. Elements
+     * Allocates an xarray_container with the specified shape and layout. Elements
      * are initialized to the specified value.
-     * @param shape the shape of the xarray
+     * @param shape the shape of the xarray_container
      * @param value the value of the elements
-     * @param l the layout of the xarray
+     * @param l the layout of the xarray_container
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(const shape_type& shape, const_reference value, layout l)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(const shape_type& shape, const_reference value, layout l)
         : base_type()
     {
         base_type::reshape(shape, l);
@@ -235,26 +237,26 @@ namespace xt
     }
 
     /**
-     * Allocates an uninitialized xarray with the specified shape and strides.
-     * @param shape the shape of the xarray
-     * @param strides the strides of the xarray
+     * Allocates an uninitialized xarray_container with the specified shape and strides.
+     * @param shape the shape of the xarray_container
+     * @param strides the strides of the xarray_container
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(const shape_type& shape, const strides_type& strides)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(const shape_type& shape, const strides_type& strides)
         : base_type()
     {
         base_type::reshape(shape, strides);
     }
 
     /**
-     * Allocates an uninitialized xarray with the specified shape and strides.
+     * Allocates an uninitialized xarray_container with the specified shape and strides.
      * Elements are initialized to the specified value.
-     * @param shape the shape of the xarray
-     * @param strides the strides of the xarray
+     * @param shape the shape of the xarray_container
+     * @param strides the strides of the xarray_container
      * @param value the value of the elements
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(const shape_type& shape, const strides_type& strides, const_reference value)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(const shape_type& shape, const strides_type& strides, const_reference value)
         : base_type()
     {
         base_type::reshape(shape, strides);
@@ -262,12 +264,12 @@ namespace xt
     }
 
     /**
-     * Allocates an xarray that holds a single element initialized to the
+     * Allocates an xarray_container that holds a single element initialized to the
      * specified value.
      * @param t the value of the element
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(const value_type& t)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(const value_type& t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -280,11 +282,11 @@ namespace xt
      */
     //@{
     /**
-     * Allocates a one-dimensional xarray.
-     * @param t the elements of the xarray
+     * Allocates a one-dimensional xarray_container.
+     * @param t the elements of the xarray_container
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(nested_initializer_list_t<T, 1> t)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(nested_initializer_list_t<value_type, 1> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -292,11 +294,11 @@ namespace xt
     }
 
     /**
-     * Allocates a two-dimensional xarray.
-     * @param t the elements of the xarray
+     * Allocates a two-dimensional xarray_container.
+     * @param t the elements of the xarray_container
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(nested_initializer_list_t<T, 2> t)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(nested_initializer_list_t<value_type, 2> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -304,11 +306,11 @@ namespace xt
     }
 
     /**
-     * Allocates a three-dimensional xarray.
-     * @param t the elements of the xarray
+     * Allocates a three-dimensional xarray_container.
+     * @param t the elements of the xarray_container
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(nested_initializer_list_t<T, 3> t)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(nested_initializer_list_t<value_type, 3> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -316,11 +318,11 @@ namespace xt
     }
 
     /**
-     * Allocates a four-dimensional xarray.
-     * @param t the elements of the xarray
+     * Allocates a four-dimensional xarray_container.
+     * @param t the elements of the xarray_container
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(nested_initializer_list_t<T, 4> t)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(nested_initializer_list_t<value_type, 4> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -328,11 +330,11 @@ namespace xt
     }
 
     /**
-     * Allocates a five-dimensional xarray.
-     * @param t the elements of the xarray
+     * Allocates a five-dimensional xarray_container.
+     * @param t the elements of the xarray_container
      */
-    template <class T, class EA, class SA>
-    inline xarray<T, EA, SA>::xarray(nested_initializer_list_t<T, 5> t)
+    template <class EC, class SC>
+    inline xarray_container<EC, SC>::xarray_container(nested_initializer_list_t<value_type, 5> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -348,9 +350,9 @@ namespace xt
     /**
      * The extended copy constructor.
      */
-    template <class T, class EA, class SA>
+    template <class EC, class SC>
     template <class E>
-    inline xarray<T, EA, SA>::xarray(const xexpression<E>& e)
+    inline xarray_container<EC, SC>::xarray_container(const xexpression<E>& e)
         : base_type()
     {
         semantic_base::assign(e);
@@ -359,22 +361,22 @@ namespace xt
     /**
      * The extended assignment operator.
      */
-    template <class T, class EA, class SA>
+    template <class EC, class SC>
     template <class E>
-    inline auto xarray<T, EA, SA>::operator=(const xexpression<E>& e) -> self_type&
+    inline auto xarray_container<EC, SC>::operator=(const xexpression<E>& e) -> self_type&
     {
         return semantic_base::operator=(e);
     }
     //@}
 
-    template <class T, class EA, class SA>
-    inline auto xarray<T, EA, SA>::data_impl() noexcept -> container_type&
+    template <class EC, class SC>
+    inline auto xarray_container<EC, SC>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
-    template <class T, class EA, class SA>
-    inline auto xarray<T, EA, SA>::data_impl() const noexcept -> const container_type&
+    template <class EC, class SC>
+    inline auto xarray_container<EC, SC>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
@@ -391,8 +393,8 @@ namespace xt
      * Constructs an xarray_adaptor of the given stl-like container.
      * @param data the container to adapt
      */
-    template <class C, class EA, class SA>
-    inline xarray_adaptor<C, EA, SA>::xarray_adaptor(container_type& data)
+    template <class EC, class SC>
+    inline xarray_adaptor<EC, SC>::xarray_adaptor(container_type& data)
         : base_type(), m_data(data)
     {
     }
@@ -404,8 +406,8 @@ namespace xt
      * @param shape the shape of the xarray_adaptor
      * @param l the layout of the xarray_adaptor
      */
-    template <class C, class EA, class SA>
-    inline xarray_adaptor<C, EA, SA>::xarray_adaptor(container_type& data, const shape_type& shape, layout l)
+    template <class EC, class SC>
+    inline xarray_adaptor<EC, SC>::xarray_adaptor(container_type& data, const shape_type& shape, layout l)
         : base_type(), m_data(data)
     {
         base_type::reshape(shape, l);
@@ -418,24 +420,24 @@ namespace xt
      * @param shape the shape of the xarray_adaptor
      * @param strides the strides of the xarray_adaptor
      */
-    template <class C, class EA, class SA>
-    inline xarray_adaptor<C, EA, SA>::xarray_adaptor(container_type& data, const shape_type& shape, const strides_type& strides)
+    template <class EC, class SC>
+    inline xarray_adaptor<EC, SC>::xarray_adaptor(container_type& data, const shape_type& shape, const strides_type& strides)
         : base_type(), m_data(data)
     {
         base_type::reshape(shape, strides);
     }
     //@}
 
-    template <class C, class EA, class SA>
-    inline xarray_adaptor<C, EA, SA>& xarray_adaptor<C, EA, SA>::operator=(const xarray_adaptor& rhs)
+    template <class EC, class SC>
+    inline auto xarray_adaptor<EC, SC>::operator=(const xarray_adaptor& rhs) -> self_type&
     {
         base_type::operator=(rhs);
         m_data = rhs.m_data;
         return *this;
     }
 
-    template <class C, class EA, class SA>
-    inline xarray_adaptor<C, EA, SA>& xarray_adaptor<C, EA, SA>::operator=(xarray_adaptor&& rhs)
+    template <class EC, class SC>
+    inline auto xarray_adaptor<EC, SC>::operator=(xarray_adaptor&& rhs) -> self_type&
     {
         base_type::operator=(std::move(rhs));
         m_data = rhs.m_data;
@@ -449,28 +451,28 @@ namespace xt
     /**
      * The extended assignment operator.
      */
-    template <class C, class EA, class SA>
+    template <class EC, class SC>
     template <class E>
-    inline auto xarray_adaptor<C, EA, SA>::operator=(const xexpression<E>& e) -> self_type&
+    inline auto xarray_adaptor<EC, SC>::operator=(const xexpression<E>& e) -> self_type&
     {
         return semantic_base::operator=(e);
     }
     //@}
 
-    template <class C, class EA, class SA>
-    inline auto xarray_adaptor<C, EA, SA>::data_impl() noexcept -> container_type&
+    template <class EC, class SC>
+    inline auto xarray_adaptor<EC, SC>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
-    template <class C, class EA, class SA>
-    inline auto xarray_adaptor<C, EA, SA>::data_impl() const noexcept -> const container_type&
+    template <class EC, class SC>
+    inline auto xarray_adaptor<EC, SC>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
 
-    template <class C, class EA, class SA>
-    inline void xarray_adaptor<C, EA, SA>::assign_temporary_impl(temporary_type& tmp)
+    template <class EC, class SC>
+    inline void xarray_adaptor<EC, SC>::assign_temporary_impl(temporary_type& tmp)
     {
         // TODO (performance improvement) : consider moving tmps
         // shape and strides
