@@ -105,13 +105,8 @@ namespace xt
                 return access_impl(args...);
             }
 
-            inline T operator[](const xindex& idx) const
-            {
-                return m_start + m_step * T(idx[0]);
-            }
-
             template <class It>
-            inline T element(It first, It /*last*/) const
+            inline T element(It first, It) const
             {
                 return m_start + m_step * T(*first);
             }
@@ -122,7 +117,7 @@ namespace xt
             value_type m_step;
 
             template <class T1, class... Args>
-            inline T access_impl(T1 t, Args... /*args*/) const
+            inline T access_impl(T1 t, Args...) const
             {
                 return m_start + m_step * T(t);
             }
@@ -143,12 +138,6 @@ namespace xt
             {
             }
 
-            inline value_type operator()() const
-            {
-                // special case when called without args (happens when printing)
-                return value_type();
-            }
-
             template <class... Args>
             inline value_type operator()(Args... args) const
             {
@@ -156,11 +145,6 @@ namespace xt
                 return access_impl(std::begin(idx), std::end(idx));
             }
 
-            inline value_type operator[](const xindex& idx) const
-            {
-                return access_impl(idx.begin(), idx.end());
-            }
-
             template <class It>
             inline value_type element(It first, It last) const
             {
@@ -302,17 +286,14 @@ namespace xt
             template <class... Args>
             inline value_type operator()(Args... args) const
             {
+                // TODO: avoid memory allocation
                 return access_impl(xindex({{static_cast<size_type>(args)...}}));
             }
 
-            inline value_type operator[](const xindex& idx) const
-            {
-                return access_impl(idx);
-            }
-
             template <class It>
             inline value_type element(It first, It last) const
             {
+                // TODO: avoid memory allocation
                 return access_impl(xindex(first, last));
             }
 
@@ -320,7 +301,8 @@ namespace xt
 
             inline value_type access_impl(xindex idx) const
             {
-                auto match = [this, &idx](auto& arr) {
+                auto match = [this, &idx](auto& arr)
+                {
                     if (idx[this->m_axis] >= arr.shape()[this->m_axis])
                     {
                         idx[this->m_axis] -= arr.shape()[this->m_axis];
@@ -329,7 +311,8 @@ namespace xt
                     return true;
                 };
 
-                auto get = [&idx](auto& arr) {
+                auto get = [&idx](auto& arr)
+                {
                     return arr[idx];
                 };
 
@@ -362,17 +345,14 @@ namespace xt
             template <class... Args>
             inline value_type operator()(Args... args) const
             {
+                // TODO: avoid memory allocation
                 return access_impl(xindex({{static_cast<size_type>(args)...}}));
             }
 
-            inline value_type operator[](const xindex& idx) const
-            {
-                return access_impl(idx);
-            }
-
             template <class It>
             inline value_type element(It first, It last) const
             {
+                // TODO: avoid memory allocation
                 return access_impl(xindex(first, last));
             }
 
@@ -380,7 +360,8 @@ namespace xt
 
             inline value_type access_impl(xindex idx) const
             {
-                auto get_item = [&idx](auto& arr) {
+                auto get_item = [&idx](auto& arr)
+                {
                     return arr[idx];
                 };
                 std::size_t i = idx[m_axis];
@@ -407,24 +388,14 @@ namespace xt
             template <class... Args>
             value_type operator()(Args... args) const
             {
-                // to catch a -Wuninitialized
-                if (sizeof...(Args))
-                {
-                    std::array<size_type, sizeof...(Args)> args_arr({static_cast<size_type>(args)...});
-                    return m_source(args_arr[m_axis]);
-                }
-                return m_source();
-            }
-
-            value_type operator[](const xindex& idx) const
-            {
-                return m_source[{ idx[m_axis] }];
+                std::array<size_type, sizeof...(Args)> args_arr({static_cast<size_type>(args)...});
+                return m_source(args_arr[m_axis]);
             }
 
             template <class It>
             inline value_type element(It first, It) const
             {
-                return m_source[{*(first + m_axis)}];
+                return m_source(*(first + m_axis));
             }
 
         private:
@@ -441,7 +412,7 @@ namespace xt
     template <class... Types>
     inline auto xtuple(Types&&... args)
     {
-        return std::tuple<detail::const_closure_t<Types>...>(std::forward<Types>(args)...);
+        return std::tuple<const_closure_t<Types>...>(std::forward<Types>(args)...);
     }
 
     /**
@@ -556,5 +527,288 @@ namespace xt
     {
         return detail::meshgrid_impl(std::make_index_sequence<sizeof...(E)>(), std::forward<E>(e)...);
     }
+
+    namespace detail
+    {
+        template <class CT>
+        struct diagonal_fn
+        {
+            using xexpression_type = std::decay_t<CT>;
+            using value_type = typename xexpression_type::value_type;
+
+            diagonal_fn(CT source, int offset, std::size_t axis_1, std::size_t axis_2)
+                : m_source(source), m_offset(offset), m_axis_1(axis_1), m_axis_2(axis_2)
+            {
+            }
+
+            template <class It>
+            inline value_type operator()(It begin, It) const
+            {
+                xindex idx(m_source.shape().size());
+
+                for (std::size_t i = 0; i < idx.size(); i++)
+                {
+                    if (i != m_axis_1 && i != m_axis_2)
+                    {
+                        idx[i] = *begin++;
+                    }
+                }
+                if (m_offset >= 0)
+                {
+                    idx[m_axis_1] = *(begin);
+                    idx[m_axis_2] = *(begin) + m_offset;
+                }
+                else
+                {
+                    idx[m_axis_1] = *(begin) - m_offset;
+                    idx[m_axis_2] = *(begin);
+                }
+                return m_source[idx];
+            }
+
+        private:
+            CT m_source;
+            const int m_offset;
+            const std::size_t m_axis_1;
+            const std::size_t m_axis_2;
+        };
+
+        template <class CT>
+        struct diag_fn
+        {
+            using xexpression_type = std::decay_t<CT>;
+            using value_type = typename xexpression_type::value_type;
+
+            diag_fn(CT source, int k) : m_source(source), m_k(k)
+            {
+            }
+
+            template <class It>
+            inline value_type operator()(It begin, It) const
+            {
+                if (m_k > 0)
+                {
+                    return *begin + m_k == *(begin + 1) ? m_source(*begin) : value_type(0);
+                }
+                else
+                {
+                    return *begin + m_k == *(begin + 1) ? m_source(*begin + m_k) : value_type(0);
+                }
+            }
+
+        private:
+            CT m_source;
+            const int m_k;
+        };
+
+        template <class CT>
+        struct flip_impl
+        {
+            using xexpression_type = std::decay_t<CT>;
+            using value_type = typename xexpression_type::value_type;
+            using size_type = typename xexpression_type::size_type;
+
+            flip_impl(CT source, std::size_t axis)
+                : m_source(source), m_axis(axis), m_shape_at_axis(m_source.shape()[m_axis] - 1)
+            {
+            }
+
+            template <class... Args>
+            inline value_type operator()(Args... args) const
+            {
+                std::array<size_type, sizeof...(Args)> idx({static_cast<size_type>(args)...});
+                return access_impl(idx.begin(), idx.end());
+            }
+
+            template <class It>
+            inline value_type element(It first, It last) const
+            {
+                // TODO: avoid memory allocation
+                xindex idx(first, last);
+                return access_impl(idx.begin(), idx.end());
+            }
+
+        private:
+            template <class It>
+            inline value_type access_impl(It begin, It end) const
+            {
+                auto it = begin + m_axis;
+                *it = m_shape_at_axis - *it;
+                return m_source.element(begin, end);
+            }
+
+            CT m_source;
+            const size_type m_axis;
+            const size_type m_shape_at_axis;
+        };
+
+        template <class CT, class Comp>
+        struct trilu_fn
+        {
+            using xexpression_type = std::decay_t<CT>;
+            using value_type = typename xexpression_type::value_type;
+            using signed_idx_type = long int;
+
+            trilu_fn(CT source, int k, Comp comp)
+                : m_source(source), m_k(k), m_comp(comp)
+            {
+            }
+
+            template <class It>
+            inline value_type operator()(It begin, It end) const
+            {
+                // have to cast to signed int otherwise -1 can lead to overflow
+                return m_comp(signed_idx_type(*begin) + m_k, signed_idx_type(*(begin + 1))) ? m_source.element(begin, end) : value_type(0);
+            }
+
+        private:
+            CT m_source;
+            const signed_idx_type m_k;
+            const Comp m_comp;
+        };
+    }
+
+    /**
+     * @brief Returns the elements on the diagonal of arr
+     * If arr has more than two dimensions, then the axes specified by 
+     * axis_1 and axis_2 are used to determine the 2-D sub-array whose 
+     * diagonal is returned. The shape of the resulting array can be 
+     * determined by removing axis1 and axis2 and appending an index 
+     * to the right equal to the size of the resulting diagonals.
+     *
+     * @param arr the input array
+     * @param offset offset of the diagonal from the main diagonal. Can
+     *               be positive or negative.
+     * @param axis_1 Axis to be used as the first axis of the 2-D sub-arrays 
+     *               from which the diagonals should be taken. 
+     * @param axis_2 Axis to be used as the second axis of the 2-D sub-arrays 
+     *               from which the diagonals should be taken. 
+     * @returns xexpression with values of the diagonal
+     *
+     * \code{.cpp}
+     * xt::xarray<double> a = {{1, 2, 3},
+     *                         {4, 5, 6}
+     *                         {7, 8, 9}};
+     * auto b = xt::diagonal(a); // => {1, 5, 9}
+     * \endcode
+     */
+    template <class E>
+    inline auto diagonal(E&& arr, int offset = 0, std::size_t axis_1 = 0, std::size_t axis_2 = 1)
+    {
+        using CT = xclosure_t<E>;
+        auto shape = arr.shape();
+
+        // the following shape calculation code is an almost verbatim adaptation of numpy:
+        // https://github.com/numpy/numpy/blob/2aabeafb97bea4e1bfa29d946fbf31e1104e7ae0/numpy/core/src/multiarray/item_selection.c#L1799
+
+        auto ret_shape = std::vector<std::size_t>(arr.dimension());
+
+        std::size_t dim_1 = shape[axis_1];
+        std::size_t dim_2 = shape[axis_2];
+
+        std::size_t n_dim = arr.dimension();
+
+        offset >= 0 ? dim_2 -= offset : dim_1 += offset;
+
+        auto diag_size = dim_2 < dim_1 ? dim_2 : dim_1;
+        if (diag_size < 0)
+        {
+            diag_size = 0;
+        }
+
+        std::size_t i = 0;
+        for (std::size_t idim = 0; idim < n_dim; ++idim)
+        {
+            if (idim != axis_1 && idim != axis_2)
+            {
+                ret_shape[i++] = shape[idim];
+            }
+        }
+
+        ret_shape[n_dim - 2] = diag_size;
+        ret_shape.pop_back();
+
+        return detail::make_xgenerator(detail::fn_impl<detail::diagonal_fn<CT>>(detail::diagonal_fn<CT>(std::forward<E>(arr), offset, axis_1, axis_2)),
+                                       ret_shape);
+    }
+
+    /**
+     * @brief xexpression with values of arr on the diagonal, zeroes otherwise
+     *
+     * @param arr the 1D input array of length n
+     * @returns xexpression function with shape n x n and arr on the diagonal
+     *
+     * \code{.cpp}
+     * xt::xarray<double> a = {1, 5, 9};
+     * auto b = xt::diag(a); // => {{1, 0, 0},
+     *                       //     {0, 5, 0},
+     *                       //     {0, 0, 9}}
+     * \endcode
+     */
+    template <class E>
+    inline auto diag(E&& arr, int k = 0)
+    {
+        using CT = xclosure_t<E>;
+        std::size_t s = arr.shape()[0] + std::abs(k);
+        return detail::make_xgenerator(detail::fn_impl<detail::diag_fn<CT>>(detail::diag_fn<CT>(std::forward<E>(arr), k)),
+                                       { s, s });
+    }
+
+    /**
+     * @brief Reverse the order of elements in an xexpression along the given axis.
+     * Note: A NumPy/Matlab style `flipud(arr)` is equivalent to `xt::flip(arr, 0)`,
+     * `fliplr(arr)` to `xt::flip(arr, 1)`.
+     * 
+     * @param arr the input xexpression
+     * @param axis the axis along which elements should be reversed
+     *
+     * @return xexpression evaluating to reversed array
+     */
+    template <class E>
+    inline auto flip(E&& arr, std::size_t axis)
+    {
+        using CT = xclosure_t<E>;
+        auto shape = arr.shape();
+        return detail::make_xgenerator(detail::flip_impl<CT>(std::forward<E>(arr), axis),
+                                       shape);
+    }
+
+    /**
+     * @brief Extract lower triangular matrix from xexpression. The parameter k selects the
+     *        offset of the diagonal.
+     *
+     * @param arr the input array
+     * @param k the diagonal above which to zero elements. 0 (default) selects the main diagonal,
+     *          k < 0 is below the main diagonal, k > 0 above.
+     * @returns xexpression containing lower triangle from arr, 0 otherwise
+     */
+    template <class E>
+    inline auto tril(E&& arr, int k = 0)
+    {
+        using CT = xclosure_t<E>;
+        auto shape = arr.shape();
+        return detail::make_xgenerator(detail::fn_impl<detail::trilu_fn<CT, std::greater_equal<long int>>>(
+                                       detail::trilu_fn<CT, std::greater_equal<long int>>(std::forward<E>(arr), k, std::greater_equal<long int>())),
+                                       shape);
+    }
+
+    /**
+     * @brief Extract upper triangular matrix from xexpression. The parameter k selects the
+     *        offset of the diagonal.
+     *
+     * @param arr the input array
+     * @param k the diagonal below which to zero elements. 0 (default) selects the main diagonal,
+     *          k < 0 is below the main diagonal, k > 0 above.
+     * @returns xexpression containing lower triangle from arr, 0 otherwise
+     */
+    template <class E>
+    inline auto triu(E&& arr, int k = 0)
+    {
+        using CT = xclosure_t<E>;
+        auto shape = arr.shape();
+        return detail::make_xgenerator(detail::fn_impl<detail::trilu_fn<CT, std::less_equal<long int>>>(
+                                       detail::trilu_fn<CT, std::less_equal<long int>>(std::forward<E>(arr), k, std::less_equal<long int>())),
+                                       shape);
+    }
 }
-#endif
+#endif
\ No newline at end of file
