@@ -25,44 +25,45 @@ namespace xt
      * xtensor declaration *
      ***********************/
 
-    template <class T, std::size_t N, class A>
-    struct xcontainer_inner_types<xtensor<T, N, A>>
+    template <class EC, std::size_t N>
+    struct xcontainer_inner_types<xtensor_container<EC, N>>
     {
-        using container_type = std::vector<T, A>;
+        using container_type = EC;
         using shape_type = std::array<typename container_type::size_type, N>;
         using strides_type = shape_type;
         using backstrides_type = shape_type;
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
-        using temporary_type = xtensor<T, N, A>;
+        using temporary_type = xtensor_container<EC, N>;
     };
 
-    template <class T, std::size_t N, class A>
-    struct xiterable_inner_types<xtensor<T, N, A>>
-        : xcontainer_iterable_types<xtensor<T, N, A>>
+    template <class EC, std::size_t N>
+    struct xiterable_inner_types<xtensor_container<EC, N>>
+        : xcontainer_iterable_types<xtensor_container<EC, N>>
     {
     };
 
     /**
-     * @class xtensor
-     * @brief Dense multidimensional container with tensor
-     * semantic and fixed dimension.
+     * @class xtensor_container
+     * @brief Dense multidimensional container with tensor semantic and fixed
+     * dimension.
      *
-     * The xtensor class implements a dense multidimensional container
+     * The xtensor_container class implements a dense multidimensional container
      * with tensor semantic and fixed dimension
      *
-     * @tparam T The type of objects stored in the container.
+     * @tparam EC The type of the container holding the elements.
      * @tparam N The dimension of the container.
+     * @sa xtensor
      */
-    template <class T, size_t N, class A>
-    class xtensor : public xstrided_container<xtensor<T, N, A>>,
-                    public xcontainer_semantic<xtensor<T, N, A>>
+    template <class EC, size_t N>
+    class xtensor_container : public xstrided_container<xtensor_container<EC, N>>,
+                              public xcontainer_semantic<xtensor_container<EC, N>>
     {
 
     public:
 
-        using self_type = xtensor<T, N, A>;
+        using self_type = xtensor_container<EC, N>;
         using base_type = xstrided_container<self_type>;
         using semantic_base = xcontainer_semantic<self_type>;
         using container_type = typename base_type::container_type;
@@ -74,26 +75,26 @@ namespace xt
         using shape_type = typename base_type::shape_type;
         using strides_type = typename base_type::strides_type;
 
-        xtensor();
-        xtensor(nested_initializer_list_t<T, N> t);
-        explicit xtensor(const shape_type& shape, layout l = layout::row_major);
-        explicit xtensor(const shape_type& shape, const_reference value, layout l = layout::row_major);
-        explicit xtensor(const shape_type& shape, const strides_type& strides);
-        explicit xtensor(const shape_type& shape, const strides_type& strides, const_reference value);
+        xtensor_container();
+        xtensor_container(nested_initializer_list_t<value_type, N> t);
+        explicit xtensor_container(const shape_type& shape, layout l = layout::row_major);
+        explicit xtensor_container(const shape_type& shape, const_reference value, layout l = layout::row_major);
+        explicit xtensor_container(const shape_type& shape, const strides_type& strides);
+        explicit xtensor_container(const shape_type& shape, const strides_type& strides, const_reference value);
 
-        ~xtensor() = default;
+        ~xtensor_container() = default;
 
-        xtensor(const xtensor&) = default;
-        xtensor& operator=(const xtensor&) = default;
+        xtensor_container(const xtensor_container&) = default;
+        xtensor_container& operator=(const xtensor_container&) = default;
 
-        xtensor(xtensor&&) = default;
-        xtensor& operator=(xtensor&&) = default;
+        xtensor_container(xtensor_container&&) = default;
+        xtensor_container& operator=(xtensor_container&&) = default;
 
         template <class E>
-        xtensor(const xexpression<E>& e);
+        xtensor_container(const xexpression<E>& e);
 
         template <class E>
-        xtensor& operator=(const xexpression<E>& e);
+        xtensor_container& operator=(const xexpression<E>& e);
 
     private:
 
@@ -102,56 +103,56 @@ namespace xt
         container_type& data_impl() noexcept;
         const container_type& data_impl() const noexcept;
 
-        friend class xcontainer<xtensor<T, N, A>>;
+        friend class xcontainer<xtensor_container<EC, N>>;
     };
 
-    /*******************************
-     * xtensor_adaptor declaration *
-     *******************************/
+    /*****************************************
+     * xtensor_container_adaptor declaration *
+     *****************************************/
 
-    template <class C, std::size_t N, class A = std::allocator<typename C::value_type>>
+    template <class EC, std::size_t N>
     class xtensor_adaptor;
 
-    template <class C, std::size_t N, class A>
-    struct xcontainer_inner_types<xtensor_adaptor<C, N, A>>
+    template <class EC, std::size_t N>
+    struct xcontainer_inner_types<xtensor_adaptor<EC, N>>
     {
-        using container_type = C;
+        using container_type = EC;
         using shape_type = std::array<typename container_type::size_type, N>;
         using strides_type = shape_type;
         using backstrides_type = shape_type;
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
-        using temporary_type = xtensor<typename C::value_type, N, A>;
+        using temporary_type = xtensor_container<EC, N>;
     };
 
-    template <class C, std::size_t N, class A>
-    struct xiterable_inner_types<xtensor_adaptor<C, N, A>>
-        : xcontainer_iterable_types<xtensor_adaptor<C, N, A>>
+    template <class EC, std::size_t N>
+    struct xiterable_inner_types<xtensor_adaptor<EC, N>>
+        : xcontainer_iterable_types<xtensor_adaptor<EC, N>>
     {
     };
 
     /**
      * @class xtensor_adaptor
-     * @brief Dense multidimensional container adaptor with
-     * tensor semantic and fixed dimension.
+     * @brief Dense multidimensional container adaptor with tensor semantic
+     * and fixed dimension.
      *
      * The xtensor_adaptor class implements a dense multidimensional
      * container adaptor with tensor semantic and fixed dimension. It
      * is used to provide a multidimensional container semantic and a
      * tensor semantic to stl-like containers.
      *
-     * @tparam C The container type to adapt.
+     * @tparam EC The container type to adapt.
      * @tparam N The dimension of the adaptor.
      */
-    template <class C, std::size_t N, class A>
-    class xtensor_adaptor : public xstrided_container<xtensor_adaptor<C, N, A>>,
-                            public xadaptor_semantic<xtensor_adaptor<C, N, A>>
+    template <class EC, std::size_t N>
+    class xtensor_adaptor : public xstrided_container<xtensor_adaptor<EC, N>>,
+                            public xadaptor_semantic<xtensor_adaptor<EC, N>>
     {
 
     public:
 
-        using self_type = xtensor_adaptor<C, N, A>;
+        using self_type = xtensor_adaptor<EC, N>;
         using base_type = xstrided_container<self_type>;
         using semantic_base = xadaptor_semantic<self_type>;
         using container_type = typename base_type::container_type;
@@ -183,32 +184,32 @@ namespace xt
         using temporary_type = typename xcontainer_inner_types<self_type>::temporary_type;
         void assign_temporary_impl(temporary_type& tmp);
 
-        friend class xcontainer<xtensor_adaptor<C, N, A>>;
-        friend class xadaptor_semantic<xtensor_adaptor<C, N, A>>;
+        friend class xcontainer<xtensor_adaptor<EC, N>>;
+        friend class xadaptor_semantic<xtensor_adaptor<EC, N>>;
     };
 
-    /**************************
-     * xtensor implementation *
-     **************************/
+    /************************************
+     * xtensor_container implementation *
+     ************************************/
 
      /**
       * @name Constructors
       */
      //@{
      /**
-      * Allocates an uninitialized xtensor that holds 0 element.
+      * Allocates an uninitialized xtensor_container that holds 0 element.
       */
-    template <class T, std::size_t N, class A>
-    inline xtensor<T, N, A>::xtensor()
+    template <class EC, std::size_t N>
+    inline xtensor_container<EC, N>::xtensor_container()
         : base_type(), m_data(1, value_type())
     {
     }
 
      /**
-      * Allocates an xtensor with nested initializer lists.
+      * Allocates an xtensor_container with nested initializer lists.
       */
-    template <class T, std::size_t N, class A>
-    inline xtensor<T, N, A>::xtensor(nested_initializer_list_t<T, N> t)
+    template <class EC, std::size_t N>
+    inline xtensor_container<EC, N>::xtensor_container(nested_initializer_list_t<value_type, N> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -216,27 +217,27 @@ namespace xt
     }
 
     /**
-     * Allocates an uninitialized xtensor with the specified shape and
+     * Allocates an uninitialized xtensor_container with the specified shape and
      * layout.
-     * @param shape the shape of the xtensor
-     * @param l the layout of the xtensor
+     * @param shape the shape of the xtensor_container
+     * @param l the layout of the xtensor_container
      */
-    template <class T, std::size_t N, class A>
-    inline xtensor<T, N, A>::xtensor(const shape_type& shape, layout l)
+    template <class EC, std::size_t N>
+    inline xtensor_container<EC, N>::xtensor_container(const shape_type& shape, layout l)
         : base_type()
     {
         base_type::reshape(shape, l);
     }
 
     /**
-     * Allocates an xtensor with the specified shape and layout. Elements
+     * Allocates an xtensor_container with the specified shape and layout. Elements
      * are initialized to the specified value.
-     * @param shape the shape of the xtensor
+     * @param shape the shape of the xtensor_container
      * @param value the value of the elements
-     * @param l the layout of the xtensor
+     * @param l the layout of the xtensor_container
      */
-    template <class T, std::size_t N, class A>
-    inline xtensor<T, N, A>::xtensor(const shape_type& shape, const_reference value, layout l)
+    template <class EC, std::size_t N>
+    inline xtensor_container<EC, N>::xtensor_container(const shape_type& shape, const_reference value, layout l)
         : base_type()
     {
         base_type::reshape(shape, l);
@@ -244,26 +245,26 @@ namespace xt
     }
 
     /**
-     * Allocates an uninitialized xtensor with the specified shape and strides.
-     * @param shape the shape of the xtensor
-     * @param strides the strides of the xtensor
+     * Allocates an uninitialized xtensor_container with the specified shape and strides.
+     * @param shape the shape of the xtensor_container
+     * @param strides the strides of the xtensor_container
      */
-    template <class T, std::size_t N, class A>
-    inline xtensor<T, N, A>::xtensor(const shape_type& shape, const strides_type& strides)
+    template <class EC, std::size_t N>
+    inline xtensor_container<EC, N>::xtensor_container(const shape_type& shape, const strides_type& strides)
         : base_type()
     {
         base_type::reshape(shape, strides);
     }
 
     /**
-     * Allocates an uninitialized xtensor with the specified shape and strides.
+     * Allocates an uninitialized xtensor_container with the specified shape and strides.
      * Elements are initialized to the specified value.
-     * @param shape the shape of the xtensor
-     * @param strides the strides of the xtensor
+     * @param shape the shape of the xtensor_container
+     * @param strides the strides of the xtensor_container
      * @param value the value of the elements
      */
-    template <class T, std::size_t N, class A>
-    inline xtensor<T, N, A>::xtensor(const shape_type& shape, const strides_type& strides, const_reference value)
+    template <class EC, std::size_t N>
+    inline xtensor_container<EC, N>::xtensor_container(const shape_type& shape, const strides_type& strides, const_reference value)
         : base_type()
     {
         base_type::reshape(shape, strides);
@@ -278,9 +279,9 @@ namespace xt
     /**
      * The extended copy constructor.
      */
-    template <class T, std::size_t N, class A>
+    template <class EC, std::size_t N>
     template <class E>
-    inline xtensor<T, N, A>::xtensor(const xexpression<E>& e)
+    inline xtensor_container<EC, N>::xtensor_container(const xexpression<E>& e)
         : base_type()
     {
         semantic_base::assign(e);
@@ -289,22 +290,22 @@ namespace xt
     /**
      * The extended assignment operator.
      */
-    template <class T, std::size_t N, class A>
+    template <class EC, std::size_t N>
     template <class E>
-    inline auto xtensor<T, N, A>::operator=(const xexpression<E>& e) -> self_type&
+    inline auto xtensor_container<EC, N>::operator=(const xexpression<E>& e) -> self_type&
     {
         return semantic_base::operator=(e);
     }
     //@}
 
-    template <class T, std::size_t N, class A>
-    inline auto xtensor<T, N, A>::data_impl() noexcept -> container_type&
+    template <class EC, std::size_t N>
+    inline auto xtensor_container<EC, N>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
-    template <class T, std::size_t N, class A>
-    inline auto xtensor<T, N, A>::data_impl() const noexcept -> const container_type&
+    template <class EC, std::size_t N>
+    inline auto xtensor_container<EC, N>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
@@ -321,8 +322,8 @@ namespace xt
      * Constructs an xtensor_adaptor of the given stl-like container.
      * @param data the container to adapt
      */
-    template <class C, std::size_t N, class A>
-    inline xtensor_adaptor<C, N, A>::xtensor_adaptor(container_type& data)
+    template <class EC, std::size_t N>
+    inline xtensor_adaptor<EC, N>::xtensor_adaptor(container_type& data)
         : base_type(), m_data(data)
     {
     }
@@ -334,8 +335,8 @@ namespace xt
      * @param shape the shape of the xtensor_adaptor
      * @param l the layout of the xtensor_adaptor
      */
-    template <class C, std::size_t N, class A>
-    inline xtensor_adaptor<C, N, A>::xtensor_adaptor(container_type& data, const shape_type& shape, layout l)
+    template <class EC, std::size_t N>
+    inline xtensor_adaptor<EC, N>::xtensor_adaptor(container_type& data, const shape_type& shape, layout l)
         : base_type(), m_data(data)
     {
         base_type::reshape(shape, l);
@@ -348,24 +349,24 @@ namespace xt
      * @param shape the shape of the xtensor_adaptor
      * @param strides the strides of the xtensor_adaptor
      */
-    template <class C, std::size_t N, class A>
-    inline xtensor_adaptor<C, N, A>::xtensor_adaptor(container_type& data, const shape_type& shape, const strides_type& strides)
+    template <class EC, std::size_t N>
+    inline xtensor_adaptor<EC, N>::xtensor_adaptor(container_type& data, const shape_type& shape, const strides_type& strides)
         : base_type(), m_data(data)
     {
         base_type::reshape(shape, strides);
     }
     //@}
 
-    template <class C, std::size_t N, class A>
-    inline xtensor_adaptor<C, N, A>& xtensor_adaptor<C, N, A>::operator=(const xtensor_adaptor& rhs)
+    template <class EC, std::size_t N>
+    inline auto xtensor_adaptor<EC, N>::operator=(const xtensor_adaptor& rhs) -> self_type&
     {
         base_type::operator=(rhs);
         m_data = rhs.m_data;
         return *this;
     }
 
-    template <class C, std::size_t N, class A>
-    inline xtensor_adaptor<C, N, A>& xtensor_adaptor<C, N, A>::operator=(xtensor_adaptor&& rhs)
+    template <class EC, std::size_t N>
+    inline auto xtensor_adaptor<EC, N>::operator=(xtensor_adaptor&& rhs) -> self_type&
     {
         base_type::operator=(std::move(rhs));
         m_data = rhs.m_data;
@@ -379,28 +380,28 @@ namespace xt
     /**
      * The extended assignment operator.
      */
-    template <class C, std::size_t N, class A>
+    template <class EC, std::size_t N>
     template <class E>
-    inline auto xtensor_adaptor<C, N, A>::operator=(const xexpression<E>& e) -> self_type&
+    inline auto xtensor_adaptor<EC, N>::operator=(const xexpression<E>& e) -> self_type&
     {
         return semantic_base::operator=(e);
     }
     //@}
 
-    template <class C, std::size_t N, class A>
-    inline auto xtensor_adaptor<C, N, A>::data_impl() noexcept -> container_type&
+    template <class EC, std::size_t N>
+    inline auto xtensor_adaptor<EC, N>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
-    template <class C, std::size_t N, class A>
-    inline auto xtensor_adaptor<C, N, A>::data_impl() const noexcept -> const container_type&
+    template <class EC, std::size_t N>
+    inline auto xtensor_adaptor<EC, N>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
 
-    template <class C, std::size_t N, class A>
-    inline void xtensor_adaptor<C, N, A>::assign_temporary_impl(temporary_type& tmp)
+    template <class EC, std::size_t N>
+    inline void xtensor_adaptor<EC, N>::assign_temporary_impl(temporary_type& tmp)
     {
         // TODO (performance improvement) : consider moving tmps shape and strides
         base_type::shape_impl() = tmp.shape();
