@@ -11,7 +11,7 @@
 
 #define XTENSOR_VERSION_MAJOR 0
 #define XTENSOR_VERSION_MINOR 7
-#define XTENSOR_VERSION_PATCH 2
+#define XTENSOR_VERSION_PATCH 3
 
 // DETECT 3.6 <= clang < 3.8 for compiler bug workaround.
 #ifdef __clang__
@@ -22,4 +22,13 @@
     #endif
 #endif
 
+#ifndef DEFAULT_DATA_CONTAINER
+#define DEFAULT_DATA_CONTAINER(T, A) std::vector<T, A>
+#endif
+
+#ifndef DEFAULT_SHAPE_CONTAINER
+#define DEFAULT_SHAPE_CONTAINER(T, EA, SA) \
+    std::vector<typename DEFAULT_DATA_CONTAINER(T, EA)::size_type, SA>
+#endif
+
 #endif
